"""
File with examples.
"""

from rxjwt.utils import generate_secret
from rxjwt.token import AccessToken

if __name__ == "__main__":
    SECRET = generate_secret()
    token = str(AccessToken.generate(payload={"a": "b"}, secret=SECRET))
    print(AccessToken(token).verify(secret=SECRET))


    class UserAccessToken(AccessToken):

        @classmethod
        def for_user(cls, user_id, secret=SECRET):
            return cls.generate({"user_id": user_id}, secret)


    user_token = UserAccessToken.for_user(100)
    print(user_token, user_token.verify(SECRET), user_token.get_payload())



class InvalidAlgorithmException(Exception):
    """
    An exception of invalid hashing algorithm
    """

    def __init__(self, algorithm, algorithms):
        """
        A constructor with just sets the algorithm property (the wrong algorithm
        name).
        :param algorithm: The wrong algorithm name.
        :param algorithms: All available algorithms.
        """
        self.algorithm = algorithm
        self.algorithms = algorithms

    def __str__(self):
        """
        Returns the exception message.
        :return: An exception message which contains info about possible
        algorithms.
        """
        return "Wrong algorithm: {algorithm}. The algorithm must be one of the" \
               "following: {algorithms}".format(algorithm=self.algorithm,
                                                algorithms=",".join(self.algorithms.keys()))


class IncorrectTokenHeaderException(Exception):
    pass


class IncorrectTokenPayloadException(Exception):
    pass



from django.conf import settings
from rest_framework.authentication import BaseAuthentication
from rest_framework.exceptions import AuthenticationFailed

from rxjwt.token import UserAccessToken as DefaultUserAceessToken
from rxjwt.utils import import_class

try:
    UserAccessToken = import_class(settings.RXJWT["ACCESS_TOKEN_CLASS"])
except (ImportError, KeyError):
    UserAccessToken = DefaultUserAceessToken


class JWTAuthentication(BaseAuthentication):

    def authenticate(self, request):
        auth_header = request.META.get("HTTP_AUTHORIZATION")
        if auth_header is None:
            return None
        if not len(auth_header.split()):
            raise AuthenticationFailed({"error": "invalid-header",
                                        "detail": "Invalid authentication header."})
        auth_type, auth_token = auth_header.split()
        if auth_type != "Token":
            raise AuthenticationFailed({"error": "invalid-header",
                                        "detail": "Invalid authentication header."})
        user_access_token = UserAccessToken(auth_token)
        if not user_access_token.verify(settings.SECRET_KEY):
            raise AuthenticationFailed(
                {"error": "invalid-token",
                 "detail": "The token is either invalid or expired, or the token's"
                           " owner is not found."})
        current_user = user_access_token.get_user()
        try:
            soft_delete_attr = settings.RXJWT["SOFT_DELETE_ATTR"]
        except KeyError:
            soft_delete_attr = "datetime_removed"
        if hasattr(current_user, soft_delete_attr) and getattr(current_user,
                                                               soft_delete_attr) is not None:
            raise AuthenticationFailed(
                {"error": "invalid-token",
                 "detail": "The token is either invalid or expired, or the token's"
                           " owner is not found."})
        current_user.save()
        return current_user, str(user_access_token)



from base64 import urlsafe_b64encode
from importlib import import_module
from uuid import uuid4


def string_as_base64(string):
    """
    Converts a utf-8 string to url save base64.
    :param string: The string to convert.
    :return: Bytes containing base64-encoded input string.
    """
    return urlsafe_b64encode(bytes(string, "utf-8"))


def generate_secret():
    """
    Generates new server secret value.
    :return: The secret value which will be used in token creation.
    """
    return bytes(str(uuid4()), "utf-8")


def import_class(class_name):
    attr = class_name.split(".")[-1]
    module = class_name.replace(".%s" % attr, "")
    return getattr(import_module(module), attr)



from distutils.core import setup

setup(
    name='rxjwt',
    packages=['rxjwt'],
    version='1.1',
    license='MIT',
    description='RXJWT - Extendable JWT library for Django REST Framework. '
                'This library provides user authorization functions through '
                'JSON Web Token (JWT) mechanism.',
    author='Mikhail Kormanowsky',
    author_email='kormanowsky@gmail.com',
    url='https://github.com/kormanowsky/rxjwt',
    download_url='https://github.com/kormanowsky/rxjwt/archive/refs/tags/v1.1.tar.gz',
    keywords=['Django', 'django-rest', 'django-auth'],
    install_requires=[
        'Django',
        'djangorestframework',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
